@@ -12,7 +12,7 @@
 #' @param field_name \code{character(1)}. Name of the field(s)
 #' @param dec_symbol \code{character(1)}. The symbol in the field used to 
 #'   denote a decimal. 
-#' @param n_dec \code{integeris(1)}. The number of decimal places permitted
+#' @param n_dec \code{integerish(1)}. The number of decimal places permitted
 #'   by the field validation. 
 #' @param checked \code{character}. Values to recognize as checked in a 
 #'   checkbox field.
@@ -308,7 +308,8 @@ raw_cast <- list(
   select             = NA,
   radio              = NA,
   dropdown           = NA,
-  sql                = NA
+  sql                = NA, 
+  system             = NA
 )
 
 #####################################################################
@@ -360,7 +361,8 @@ raw_cast <- list(
   select                   = castLabel,
   radio                    = castLabel,
   dropdown                 = castLabel,
-  sql                      = NA
+  sql                      = NA, 
+  system                   = castLabel
 )
 
 #####################################################################
@@ -425,7 +427,8 @@ raw_cast <- list(
   phone                    = as.character,
   zipcode                  = as.character, 
   slider                   = as.numeric,
-  sql                      = NA
+  sql                      = NA, 
+  system                   = castRaw
 )
 
 FIELD_TYPES <- c(
