@@ -15,10 +15,10 @@
 #'   the value will be changed to \code{'ids'}.
 #' @param returnData Logical.  Prevents the REDCap import and instead
 #'   returns the data frame that would have been given
-#'   for import.  This is sometimes helpful if the API import fails without
+#'   for import. This is sometimes helpful if the API import fails without
 #'   providing an informative message. The data frame can be written to a csv
 #'   and uploaded using the interactive tools to troubleshoot the
-#'   problem.  Please shoot me an e-mail if you find errors I havne't
+#'   problem. Please send us an e-mail if you find errors I haven't
 #'   accounted for.
 #' @param logfile An optional filepath (preferably .txt) in which to print the
 #'   log of errors and warnings about the data.
@@ -246,6 +246,7 @@ importRecords.redcapApiConnection <- function(rcon,
   
   # Remove calculated fields
   calc_field <- MetaData$field_name[MetaData$field_type == "calc"]
+  calc_field <- calc_field[calc_field %in% names(data)]
   
   if (length(calc_field) > 0)
   {
