@@ -3,7 +3,7 @@
 #' 
 #' @description These functions are primarily intended to assist with testing
 #'   features of \code{redcapAPI}. Purging and restoring project data permits 
-#'   us to perform tests on different project structures without havng to 
+#'   us to perform tests on different project structures without having to 
 #'   manage multiple projects or API tokens. 
 #'   
 #'   When purging project data, many of these actions may 
@@ -30,7 +30,7 @@
 #'   via \code{importMappings}. The API doesn't support deleting 
 #'   mappings, but an import replaces the existing mappings.
 #' @param repeating_instruments A \code{data.frame} for restoring repeating instruments
-#'   configuration via \code{importRepeatingInstrumentsEvents}. The API doesn't support deleting 
+#'   configuration via \code{\link{importRepeatingInstrumentsEvents}}. The API doesn't support deleting 
 #'   repeating instruments, but an import replaces the existing instruments.
 #'   NOT YET IMPLEMENTED
 #' @param users Either \code{logical(1)} indicating if users data should be
@@ -253,7 +253,8 @@ purgeProject.redcapApiConnection <- function(object,
   
   if (records){
     RecordId <- exportRecordsTyped(object, 
-                                   fields = object$metadata()$field_name[1], 
+                                   fields = object$metadata()$field_name[1],
+                                   cast = list(system = castRaw),
                                    error_handling = error_handling, 
                                    config = config)
     
