@@ -27,8 +27,9 @@
 #'   
 #' @details
 #' \code{redcapConnection} objects will retrieve and cache various forms of 
-#' project information. This can make metadata, arms, events, instruments, fieldnames, 
-#' arm-event mappings, users, version, project information, and fileRepository available
+#' project information. This can make metadata, arms, dags, events, instruments, fieldnames, 
+#' arm-event mappings, users, version, project information, fileRepository,
+#' and repeating instruments available
 #' directly from the \code{redcapConnection} object. Take note that 
 #' the retrieval of these objects uses the default values of the respective
 #' export functions (excepting the file repository, 
@@ -150,6 +151,7 @@ redcapConnection <- function(url = getOption('redcap_api_url'),
   this_instrument <- NULL
   this_fileRepository <- NULL
   this_repeat <- NULL
+  this_dag <- NULL
   rtry <- retries
   rtry_int <- rep(retry_interval, 
                   length.out = rtry)
@@ -168,6 +170,7 @@ redcapConnection <- function(url = getOption('redcap_api_url'),
            "instrument" = exportInstruments(rc), 
            "fileRepo" = exportFileRepositoryListing(rc, recursive = TRUE),
            "repeat" = exportRepeatingInstrumentsEvents(rc),
+           "dags" = exportDags(rc),
            NULL)
   }
   
@@ -232,10 +235,16 @@ redcapConnection <- function(url = getOption('redcap_api_url'),
       flush_repeatInstrumentEvent = function() this_repeat <<- NULL,
       refresh_repeatInstrumentEvent = function() this_repeat <<- getter("repeat"),
       
+      dags = function() {if (is.null(this_dag)) this_dag <<- getter("dags"); this_dag },
+      has_dags = function() !is.null(this_dag), 
+      flush_dags = function() this_dag <<- NULL, 
+      refresh_dags = function() this_dag <<- getter("dags"),
+      
       flush_all = function(){ 
         this_metadata <<- this_arm <<- this_event <<- this_fieldname <<- 
           this_mapping <<- this_user <<- this_version <<- this_project <<- 
-          this_instrument <<- this_fileRepository <<- this_repeat <<- NULL}, 
+          this_instrument <<- this_fileRepository <<- this_repeat <<- 
+          this_dag <<- NULL}, 
       
       refresh_all = function(){
         this_metadata <<- getter("metadata")
@@ -248,7 +257,8 @@ redcapConnection <- function(url = getOption('redcap_api_url'),
         this_project <<- getter("project")
         this_instrument <<- getter("instrument")
         this_fileRepository <<- getter("fileRepo")
-        this_repeat <<- getter("this_repeat")
+        this_repeat <<- getter("repeat")
+        this_dag <<- getter("dag")
       },
       
       retries = function() rtry, 
@@ -290,6 +300,7 @@ print.redcapApiConnection <- function(x, ...){
     c("REDCap API Connection Object", 
       sprintf("Meta Data   : %s", is_cached(x$has_metadata())), 
       sprintf("Arms        : %s", is_cached(x$has_arms())), 
+      sprintf("DAGs        : %s", is_cached(x$has_dags())),
       sprintf("Events      : %s", is_cached(x$has_events())),
       sprintf("Instruments : %s", is_cached(x$has_instruments())),
       sprintf("Field Names : %s", is_cached(x$has_fieldnames())), 
@@ -331,6 +342,8 @@ print.redcapApiConnection <- function(x, ...){
 #'   Records can be read, or a \code{data.frame}. This should be the raw 
 #'   data as downloaded from the API, for instance. Using labelled or formatted
 #'   data is likely to result in errors when passed to other functions. 
+#' @param dags Either a \code{character} giving the file from which the 
+#'   Data Access Groups can be read, or a \code{data.frame}.
 #' @export
 
 offlineConnection <- function(meta_data = NULL, 
@@ -344,7 +357,8 @@ offlineConnection <- function(meta_data = NULL,
                               project_info = NULL, 
                               file_repo = NULL, 
                               repeat_instrument = NULL,
-                              records = NULL){
+                              records = NULL, 
+                              dags = NULL){
   ###################################################################
   # Argument Validation 
   coll <- checkmate::makeAssertCollection()
@@ -470,6 +484,16 @@ offlineConnection <- function(meta_data = NULL,
     add = coll
   )
   
+  checkmate::assert(
+    checkmate::check_character(x = dags, 
+                               len = 1, 
+                               null.ok = TRUE), 
+    checkmate::check_data_frame(x = dags, 
+                                null.ok = TRUE), 
+    .var.name = "dags", 
+    add = coll
+  )
+  
   checkmate::reportAssertions(coll)
   
   ###################################################################
@@ -530,6 +554,11 @@ offlineConnection <- function(meta_data = NULL,
                                   add = coll)
   }
   
+  if (is.character(dags)){
+    checkmate::assert_file_exists(x = dags, 
+                                  add = coll)
+  }
+  
   checkmate::reportAssertions(coll)
   
   ###################################################################
@@ -573,6 +602,11 @@ offlineConnection <- function(meta_data = NULL,
       validateRedcapData(data = .offlineConnection_readFile(instruments), 
                          redcap_data = REDCAP_INSTRUMENT_STRUCTURE)
     }
+  
+  this_dag <- 
+    validateRedcapData(data = .offlineConnection_readFile(dags), 
+                       redcap_data = REDCAP_DAG_STRUCTURE)
+  
   this_record <- .offlineConnection_readFile(records)
   
   rc <- 
@@ -661,6 +695,12 @@ offlineConnection <- function(meta_data = NULL,
       refresh_repeatInstrumentEvent = function(x) {this_project <<- validateRedcapData(data = .offlineConnection_readFile(x), 
                                                                                        redcap_data = REDCAP_REPEAT_INSTRUMENT_STRUCTURE)},
       
+      dags = function(){ this_dag }, 
+      has_dags = function() !is.null(this_dag), 
+      flush_dags = function() this_dag <<- NULL, 
+      refresh_dags = function(x) {this_dag <<- validateRedcapData(data = .offlineConnection_readFile(dags), 
+                                                                  redcap_data = REDCAP_DAG_STRUCTURE)},
+      
       records = function(){ this_record },
       has_records = function() !is.null(this_record),
       flush_records = function() this_record <<- NULL,
@@ -693,7 +733,8 @@ print.redcapOfflineConnection <- function(x, ...){
         sprintf("Field Names : %s", is_cached(x$has_fieldnames())), 
         sprintf("Mapping     : %s", is_cached(x$has_mapping())),
         sprintf("Repeat Inst.: %s", is_cached(x$has_repeatInstrumentEvent())),
-        sprintf("Users       : %s", is_cached(x$has_users())), 
+        sprintf("Users       : %s", is_cached(x$has_users())),
+        sprintf("DAGs        : %s", is_cached(x$has_dags())),
         sprintf("Version     : %s", is_cached(x$has_version())), 
         sprintf("Project Info: %s", is_cached(x$has_projectInformation())), 
         sprintf("File Repo   : %s", is_cached(x$has_fileRepository())))
